﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Persistence.Data;

namespace HeuristicLab.Persistence {
  internal abstract class GenericListTransformer<TSource, TTarget> : TransformerBase {
    public override bool CanTransformType(Type type) {
      return type == typeof(List<TSource>);
    }
    public override PersistenceData ToData(object o, PersistenceMapper mapper) {
      var data = new ArrayData<TTarget>(Id);
      mapper.Cache(o, data);
      data.TypeId = mapper.GetTypeId(o.GetType());

      var enumerable = (IEnumerable<TSource>)o;
      var componentIds = new List<TTarget>();
      foreach (var component in enumerable) {
        componentIds.Add(GetTarget(component, mapper));
      }
      data.Values = componentIds.ToArray();
      return data;
    }

    public override object ToObject(PersistenceData data, PersistenceMapper mapper) {
      var collection = (ArrayData<TTarget>)data;
      var type = mapper.GetType(collection.TypeId);

      var o = Activator.CreateInstance(type, true);
      mapper.Cache(data, o);
      dynamic list = o;
      foreach (var component in collection.Values) {
        list.Add(GetSource(component, mapper));
      }
      return o;
    }

    protected abstract TTarget GetTarget(TSource source, PersistenceMapper mapper);
    protected abstract TSource GetSource(TTarget target, PersistenceMapper mapper);
  }

  internal abstract class PrimitiveListTransformer<T> : GenericListTransformer<T, T> {
    protected override T GetTarget(T source, PersistenceMapper mapper) {
      return source;
    }
    protected override T GetSource(T target, PersistenceMapper mapper) {
      return target;
    }
  }

  [Transformer("C34E96D4-8890-4AA6-BF40-D8A439ABBF2E", 401)]
  internal sealed class DoubleListTransformer : PrimitiveListTransformer<double> { }

  [Transformer("B551DCEF-64FA-4AA9-87EE-0C49F44B6903", 402)]
  internal sealed class StringListTransformer : PrimitiveListTransformer<string> { }

  [Transformer("16A662E1-BB7A-4A0B-B646-6B8B6C8B89F9", 407)]
  internal sealed class ListTransformer : GenericListTransformer<object, uint> {
    public override bool CanTransformType(Type type) {
      return type.IsGenericType && (type.GetGenericTypeDefinition() == typeof(List<>));
    }

    protected override uint GetTarget(object source, PersistenceMapper mapper) {
      return mapper.GetDataId(source);
    }
    protected override object GetSource(uint target, PersistenceMapper mapper) {
      return mapper.GetObject(target);
    }
  }

  [Transformer("DEA9A14A-034C-46B6-BD96-676B90C21159", 408)]
  internal sealed class HashSetTransformer : GenericListTransformer<object, uint> {
    public override bool CanTransformType(Type type) {
      return type.IsGenericType && (type.GetGenericTypeDefinition() == typeof(HashSet<>));
    }

    protected override uint GetTarget(object source, PersistenceMapper mapper) {
      return mapper.GetDataId(source);
    }
    protected override object GetSource(uint target, PersistenceMapper mapper) {
      return mapper.GetObject(target);
    }
  }
}


//using System;
//using System.Collections.Generic;
//using HeuristicLab.Persistence.Data;

//namespace HeuristicLab.Persistence {
//  internal abstract class ListTransformer<T> : TransformerBase {
//    public override bool CanTransformType(Type type) {
//      return type == typeof(List<T>);
//    }
//    public override PersistenceData ToData(object o, PersistenceMapper mapper) {
//      var list = (List<T>)o;
//      var data = new ArrayData<T>(Id);
//      mapper.Cache(o, data);
//      data.Values = list.ToArray();
//      return data;
//    }

//    public override object ToObject(PersistenceData data, PersistenceMapper mapper) {
//      var array = (ArrayData<T>)data;
//      var o = new List<T>();
//      mapper.Cache(data, o);
//      o.AddRange(array.Values);
//      return o;
//    }
//  }

//  [Transformer("C34E96D4-8890-4AA6-BF40-D8A439ABBF2E", 401)]
//  internal sealed class DoubleListTransformer : ListTransformer<double> { }

//  [Transformer("B551DCEF-64FA-4AA9-87EE-0C49F44B6903", 402)]
//  internal sealed class StringListTransformer : ListTransformer<string> { }
//}
